@@ -8,12 +8,17 @@ ancovaClass <- R6::R6Class(
         #### Member variables ----
         .model = NA,
         .postHocRows = NA,
-        .data = NA,
+        .finalData = NA,
+        .residuals = NA,
         emMeans = list(),
 
         #### Init + run functions ----
         .init=function() {
 
+            private$.finalData <- NULL
+            private$.model <- NULL
+            private$.residuals <- NULL
+
             private$.initMainTable()
 
             factors <- self$options$factors
@@ -22,8 +27,6 @@ ancovaClass <- R6::R6Class(
             if (length(factors) == 0 || length(modelTerms) == 0)
                 return()
 
-            private$.data <- private$.cleanData()
-
             private$.initContrastTables()
             private$.initPostHoc()
             private$.initEmm()
@@ -39,8 +42,7 @@ ancovaClass <- R6::R6Class(
             if (is.null(dep) || length(factors) == 0 || length(modelTerms) == 0)
                 return()
 
-            data <- private$.cleanData()
-            private$.data <- data
+            data <- self$finalData
 
             for (name in colnames(data)) {
                 column <- data[[name]]
@@ -65,6 +67,7 @@ ancovaClass <- R6::R6Class(
             private$.populatePostHoc(dataB64)
             private$.prepareEmmPlots(data)
             private$.populateEmmTables()
+            private$.populateOutputs()
         },
 
         #### Compute results ----
@@ -232,7 +235,10 @@ ancovaClass <- R6::R6Class(
                 table$addColumn(name='pbonferroni', title='p<sub>bonferroni</sub>', type='number', format='zto,pvalue', visible="(postHocCorr:bonf)")
                 table$addColumn(name='pholm', title='p<sub>holm</sub>', type='number', format='zto,pvalue', visible="(postHocCorr:holm)")
 
+                ciTitleES <- paste0(self$options$postHocEsCiWidth, '% Confidence Interval')
                 table$addColumn(name='d', title='Cohen\'s d', type='number', visible="(postHocES:d)")
+                table$addColumn(name='dlower', title='Lower', type='number', visible="(postHocES:d && postHocEsCi)", superTitle=ciTitleES)
+                table$addColumn(name='dupper', title='Upper', type='number', visible="(postHocES:d && postHocEsCi)", superTitle=ciTitleES)
 
                 combin <- expand.grid(bsLevels[rev(ph)])
                 combin <- sapply(combin, as.character, simplify = 'matrix')
@@ -269,6 +275,8 @@ ancovaClass <- R6::R6Class(
                     if (comp[[i]][[3]] == TRUE)
                         table$addFormat(rowNo=i, col=1, Cell.END_GROUP)
                 }
+
+                table$setNote('note', 'Comparisons are based on estimated marginal means')
             }
             private$.postHocRows <- postHocRows
         },
@@ -387,7 +395,7 @@ ancovaClass <- R6::R6Class(
                         p <- ''
                 } else if (index == 0) {
 
-                    summ <- summary.lm(private$.model)
+                    summ <- summary.lm(self$model)
                     fstat <- summ$fstatistic
 
                     ss <- modelSS
@@ -440,13 +448,20 @@ ancovaClass <- R6::R6Class(
                     # table$setStatus('running')
 
                     emmeans::emm_options(sep = ",", parens = "a^")
-                    referenceGrid <- emmeans::emmeans(private$.model, formula)
+                    referenceGrid <- emmeans::emmeans(self$model, formula)
                     none <- summary(pairs(referenceGrid, adjust='none'))
                     tukey <- summary(pairs(referenceGrid, adjust='tukey'))
                     scheffe <- summary(pairs(referenceGrid, adjust='scheffe'))
                     bonferroni <- summary(pairs(referenceGrid, adjust='bonferroni'))
                     holm <- summary(pairs(referenceGrid, adjust='holm'))
-
+                    effSize <- as.data.frame(
+                        emmeans::eff_size(
+                            referenceGrid,
+                            sigma=sigma(self$model),
+                            edf=self$model$df.residual,
+                            level=self$options$postHocEsCiWidth/100
+                        )
+                    )
                 }) # suppressWarnings
 
                 resultRows <- lapply(strsplit(as.character(none$contrast), ' - '), function(x) strsplit(x, ','))
@@ -487,8 +502,9 @@ ancovaClass <- R6::R6Class(
                         row[['pbonferroni']] <- bonferroni[index,'p.value']
                         row[['pholm']] <- holm[index,'p.value']
 
-                        n <- nrow(private$.data)
-                        row[['d']] <- abs(row[['md']] / (sqrt(n) * row[['se']]))
+                        row[['d']] <- effSize[index, 'effect.size']
+                        row[['dlower']] <- effSize[index, 'lower.CL']
+                        row[['dupper']] <- effSize[index, 'upper.CL']
 
                         table$setRow(rowNo=i, values=row)
 
@@ -541,7 +557,7 @@ ancovaClass <- R6::R6Class(
             if ( ! self$options$homo)
                 return()
 
-            data[[".RES"]] = abs(private$.model$residuals)
+            data[[".RES"]] = abs(self$residuals)
             modelTerms <- private$.modelTerms()
             factors <- self$options$factors
 
@@ -613,10 +629,16 @@ ancovaClass <- R6::R6Class(
                 }
             }
         },
+        .populateOutputs=function() {
+            if (self$options$residsOV && self$results$residsOV$isNotFilled()) {
+                self$results$residsOV$setRowNums(rownames(self$finalData))
+                self$results$residsOV$setValues(self$residuals)
+            }
+        },
 
         #### Plot functions ----
         .qqPlot=function(image, ggtheme, theme, ...) {
-            residuals <- self$residuals
+            residuals <- rstandard(self$model)
             if (is.null(residuals))
                 return()
 
@@ -635,7 +657,7 @@ ancovaClass <- R6::R6Class(
 
             group <- self$results$emm
             emmTables <- list()
-            model <- private$.model
+            model <- self$model
 
             dep <- self$options$dep
 
@@ -658,7 +680,7 @@ ancovaClass <- R6::R6Class(
 
                     suppressMessages({
                         emmeans::emm_options(sep = ",", parens = "a^")
-                        
+
                         mm <- try(
                             emmeans::emmeans(model, formula, options=list(level=self$options$ciWidthEmm / 100), weights = weights),
                             silent = TRUE
@@ -961,48 +983,76 @@ ancovaClass <- R6::R6Class(
                     reject("Factor '{}' contains no data", factorName=factorName)
             }
 
-        },
-        .cleanData=function() {
-
-            dep <- self$options$dep
-            factors <- self$options$factors
-
-            covs <- NULL
-            if ('covs' %in% names(self$options))
-                covs <- self$options$covs
-
-            data <- self$data
-
-            if ( ! is.null(dep))
-                data[[dep]] <- jmvcore::toNumeric(data[[dep]])
-
-            for (factor in factors)
-                data[[factor]] <- as.factor(data[[factor]])
-
-            for (covariate in covs)
-                data[[covariate]] <- jmvcore::toNumeric(data[[covariate]])
-
-            data <- na.omit(data)
-
-            data
         }),
+    #### Active bindings ----
     active=list(
-        residuals=function() {
+        ready=function() {
             dep <- self$options$dep
             factors <- self$options$factors
             modelTerms <- private$.modelTerms()
 
-            if (is.null(dep) || length(factors) == 0 || length(modelTerms) == 0)
-                return(NULL)
+            return( ! is.null(dep)
+                && (length(factors) > 0 || length(modelTerms) > 0)
+                && nrow(self$data) > 0)
+        },
+        finalData=function() {
 
-            data <- private$.cleanData()
+            if (self$ready && is.null(private$.finalData)) {
 
-            formula <- jmvcore::constructFormula(dep, modelTerms)
-            formula <- stats::as.formula(formula)
-            model <- stats::aov(formula, data)
+                dep <- self$options$dep
+                factors <- self$options$factors
 
-            residuals <- rstandard(model)
-            return(residuals)
+                covs <- NULL
+                if ('covs' %in% names(self$options))
+                    covs <- self$options$covs
+
+                data <- self$data
+
+                if ( ! is.null(dep))
+                    data[[dep]] <- jmvcore::toNumeric(data[[dep]])
+
+                for (factor in factors)
+                    data[[factor]] <- as.factor(data[[factor]])
+
+                for (covariate in covs)
+                    data[[covariate]] <- jmvcore::toNumeric(data[[covariate]])
+
+                private$.finalData <- na.omit(data)
+            }
+
+            private$.finalData
+        },
+        model=function() {
+
+            if (self$ready && is.null(private$.model)) {
+
+                dep <- self$options$dep
+                factors <- self$options$factors
+                modelTerms <- private$.modelTerms()
+                data <- self$finalData
+
+                suppressWarnings({
+
+                    base::options(contrasts = c("contr.sum","contr.poly"))
+
+                    for (contrast in self$options$contrasts) {
+                        levels <- base::levels(data[[contrast$var]])
+                        stats::contrasts(data[[contrast$var]]) <- private$.createContrasts(levels, contrast$type)
+                    }
+
+                    formula <- jmvcore::constructFormula(dep, modelTerms)
+                    formula <- stats::as.formula(formula)
+
+                    private$.model <- stats::aov(formula, data)
+                })
+            }
+
+            private$.model
+        },
+        residuals=function() {
+            if (self$ready && is.null(private$.residuals))
+                private$.residuals <- residuals(self$model)
+            private$.residuals
         }
     )
 )
