@@ -119,15 +119,19 @@ ttestOneSClass <- R6::R6Class(
 
                     if ( ! isError(res)) {
 
+                        nTies <- sum(column == 0)
+                        totalRankSum <- ((n-nTies) * ((n-nTies) + 1)) / 2
+                        biSerial <- (2 * (res$statistic / totalRankSum)) - 1
+
                         ttest$setRow(rowNo=i, list(
                             "stat[wilc]"=res$statistic,
                             "p[wilc]"=res$p.value,
                             "md[wilc]"=res$estimate - testValue,
-                            "cil[wilc]"=res$conf.int[1],
-                            "ciu[wilc]"=res$conf.int[2],
-                            "es[wilc]"=d,
-                            "ciles[wilc]"=dCI[1],
-                            "ciues[wilc]"=dCI[3]))
+                            "cil[wilc]"=res$conf.int[1] - testValue,
+                            "ciu[wilc]"=res$conf.int[2] - testValue,
+                            "es[wilc]"=biSerial,
+                            "ciles[wilc]"='',
+                            "ciues[wilc]"=''))
 
                     } else {
 
@@ -280,11 +284,11 @@ ttestOneSClass <- R6::R6Class(
             if (hypothesis == 'dt' && testValue == 0)
                 table$setNote("hyp", NULL)
             else if (hypothesis == 'dt')
-                table$setNote("hyp", jmvcore::format("H\u2090 population mean \u2260 {}", testValue))
+                table$setNote("hyp", jmvcore::format("H\u2090 \u03BC \u2260 {}", testValue))
             else if (hypothesis == 'gt')
-                table$setNote("hyp", jmvcore::format("H\u2090 population mean > {}", testValue))
+                table$setNote("hyp", jmvcore::format("H\u2090 \u03BC > {}", testValue))
             else if (hypothesis == 'lt')
-                table$setNote("hyp", jmvcore::format("H\u2090 population mean < {}", testValue))
+                table$setNote("hyp", jmvcore::format("H\u2090 \u03BC < {}", testValue))
         },
         .desc=function(image, ggtheme, theme, ...) {
 
